@@ -88,8 +88,10 @@ const GObject = imports.gi.GObject;
 const XApp = imports.gi.XApp;
 const PointerTracker = imports.misc.pointerTracker;
 
+const AudioDeviceSelection = imports.ui.audioDeviceSelection;
 const SoundManager = imports.ui.soundManager;
 const BackgroundManager = imports.ui.backgroundManager;
+const Config = imports.misc.config;
 const SlideshowManager = imports.ui.slideshowManager;
 var AppletManager = imports.ui.appletManager;
 const SearchProviderManager = imports.ui.searchProviderManager;
@@ -102,9 +104,12 @@ const Overview = imports.ui.overview;
 const Expo = imports.ui.expo;
 const Panel = imports.ui.panel;
 const PlacesManager = imports.ui.placesManager;
+const PolkitAuthenticationAgent = imports.ui.polkitAuthenticationAgent;
+const KeyringPrompt = imports.ui.keyringPrompt;
 const RunDialog = imports.ui.runDialog;
 const Layout = imports.ui.layout;
 const LookingGlass = imports.ui.lookingGlass;
+const NetworkAgent = imports.ui.networkAgent;
 const NotificationDaemon = imports.ui.notificationDaemon;
 const WindowAttentionHandler = imports.ui.windowAttentionHandler;
 const CinnamonDBus = imports.ui.cinnamonDBus;
@@ -120,12 +125,12 @@ const Settings = imports.ui.settings;
 const Systray = imports.ui.systray;
 const Accessibility = imports.ui.accessibility;
 const ModalDialog = imports.ui.modalDialog;
-const {readOnlyError} = imports.ui.environment;
-const {installPolyfills} = imports.ui.overrides;
 const InputMethod = imports.misc.inputMethod;
 const ScreenRecorder = imports.ui.screenRecorder;
 const {GesturesManager} = imports.ui.gestures.gesturesManager;
 const {MonitorLabeler} = imports.ui.monitorLabeler;
+const {CinnamonPortalHandler} = imports.misc.portalHandlers;
+const {EndSessionDialog} = imports.ui.endSessionDialog;;
 
 var LAYOUT_TRADITIONAL = "traditional";
 var LAYOUT_FLIPPED = "flipped";
@@ -151,6 +156,7 @@ var messageTray = null;
 var notificationDaemon = null;
 var windowAttentionHandler = null;
 var screenRecorder = null;
+var cinnamonAudioSelectionDBusService = null;
 var cinnamonDBusService = null;
 var screenshotService = null;
 var modalCount = 0;
@@ -162,6 +168,7 @@ var xdndHandler = null;
 var statusIconDispatcher = null;
 var virtualKeyboard = null;
 var layoutManager = null;
+var networkAgent = null;
 var monitorLabeler = null;
 var themeManager = null;
 var keybindingManager = null;
@@ -188,6 +195,8 @@ var popup_rendering_actor = null;
 
 var xlet_startup_error = false;
 
+var endSessionDialog = null;
+
 var gpuOffloadHelper = null;
 var gpu_offload_supported = false;
 
@@ -274,8 +283,6 @@ function start() {
     global.logError = _logError;
     global.log = _logInfo;
 
-    installPolyfills(readOnlyError, _log);
-
     let cinnamonStartTime = new Date().getTime();
 
     log(`About to start Cinnamon (${Meta.is_wayland_compositor() ? "Wayland" : "X11"} backend)`);
@@ -309,6 +316,8 @@ function start() {
 
     Clutter.get_default_backend().set_input_method(new InputMethod.InputMethod());
 
+    new CinnamonPortalHandler();
+    cinnamonAudioSelectionDBusService = new AudioDeviceSelection.AudioDeviceSelectionDBus();
     cinnamonDBusService = new CinnamonDBus.CinnamonDBus();
     setRunState(RunState.STARTUP);
 
@@ -371,7 +380,7 @@ function start() {
 
     global.reparentActor(global.top_window_group, global.stage);
 
-    global.menuStackLength = 0;
+    global.menuStack = [];
 
     layoutManager = new Layout.LayoutManager();
 
@@ -419,6 +428,9 @@ function start() {
     windowAttentionHandler = new WindowAttentionHandler.WindowAttentionHandler();
     placesManager = new PlacesManager.PlacesManager();
 
+    if (Config.HAVE_NETWORKMANAGER)
+        networkAgent = new NetworkAgent.NetworkAgent();
+
     magnifier = new Magnifier.Magnifier();
     locatePointer = new LocatePointer.locatePointer();
 
@@ -431,9 +443,9 @@ function start() {
     _initUserSession();
     screenRecorder = new ScreenRecorder.ScreenRecorder();
 
-    // Provide the bus object for gnome-session to
-    // initiate logouts.
-    //EndSessionDialog.init();
+    PolkitAuthenticationAgent.init();
+
+    KeyringPrompt.init();
 
     _startDate = new Date();
 
@@ -1568,3 +1580,22 @@ function restartCinnamon(showOsd = false) {
 
     global.reexec_self();
 }
+
+function showEndSessionDialog(mode) {
+    if (endSessionDialog != null) {
+        global.logWarning("End session dialog already exists");
+        return;
+    }
+
+    endSessionDialog = new EndSessionDialog(mode);
+    endSessionDialog.open();
+}
+
+function closeEndSessionDialog() {
+    if (endSessionDialog == null) {
+        return;
+    }
+
+    endSessionDialog.close();
+    endSessionDialog = null;
+}
